// Copyright (c) 2023 Yuki Kimoto
// MIT License

// This source is originally copied from Perl Digest::SHA

#define sha_384_512		0
#define W64			SHA64
#define sha512			NULL
#define H0384			H01
#define H0512			H01
#define H0512224		H01
#define H0512256		H01
